from dataclasses import dataclass, field
from pathlib import Path
import pandas as pd
from itertools import islice


@dataclass(order=False)
class SciData:
    df: pd.DataFrame() = field(init=False, compare=False)
    cols: list = field(init=False, compare=False)
    time_col_name: str = field(init=False, compare=False)
    col_units: list = field(init=False, compare=False)
    sample_freq: any = field(init=False, compare=False)

    def plot_measurements(self):
        self.df.plot(subplots=True)

    def get_freq(self):
        self.sample_freq = self.df[self.time_col_name].diff().mode()


@dataclass(order=False)
class ProcessedSciData(SciData):
    df_: any = field(init=False, compare=False)
    pass


@dataclass(order=False)
class _SciDataFileBase:
    raw_data: SciData = field(init=False, compare=False, default_factory=SciData)
    header: list = field(init=False, compare=False)
    tail: list = field(init=False, compare=False)
    data_file_path: Path() = field(init=False, compare=False)
    n_head_lines: int = field(init=False, compare=False)
    n_tail_lines: int = field(init=False, compare=False)
    # cols_from_topline: any = field(init=False, compare=False)
    # dlim_white_sp: bool = field(init=False, compare=False)
    # body: list = field(init=False, compare=False)


@dataclass(order=False)
class _SciDataFileDefaultBase:
    cols_from_topline: any = field(init=True, compare=False, default='infer')
    dlim_white_sp: bool = field(init=True, compare=False, default=False)


@dataclass(order=False)
class SciDataFile(_SciDataFileDefaultBase, _SciDataFileBase):

    def parser(self):
        if self.n_head_lines > 0:
            # header parsing of the data, etc.
            with open(self.data_file_path) as f:
                self.header = list(islice(f, self.n_head_lines))
            del f
        # anything other than header is considered okay to be ready by pd
        self.raw_data.df = pd.read_csv(self.data_file_path, skiprows=self.n_head_lines, delim_whitespace=self.dlim_white_sp, header=self.cols_from_topline)
        if self.cols_from_topline == 'infer':
            self.raw_data.df.columns = self.raw_data.df.columns.str.strip()
            self.raw_data.cols = self.raw_data.df.columns.to_list()
            pass
        if self.n_tail_lines > 0:
            self.tail = self.raw_data.df.iloc[-self.n_tail_lines:].to_csv(header=None, index=None).split('\n')
            self.raw_data.df.drop(self.raw_data.df.tail(self.n_tail_lines).index, inplace=True)

# # drop na columns or na rows
# self.df.dropna(how='all', inplace=True)
# self.df.dropna(how='all', axis='columns', inplace=True)
# self.df.reset_index(inplace=True)

# def units_rfind(input_str: str, rf='('):
#     units_str = input_str[input_str.rfind(rf) + 1:-1]
#     return units_str
#
#
# def units_rfind_series(input_str_series, rf='('):
#     units_of_series = input_str_series.apply(lambda x: units_rfind(x, rf))
#     return units_of_series
